# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2011 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

################################################################################
############# IMPORTAZIONE MODULI ##############################################
################################################################################
import gtk
import os
from preventares.preventaresconfig import get_user_path
from preventares.preventaresconfig import get_data_path
from preventares.helpers import get_builder
from preventares import preventa_print
################################################################################
############ SUPPORTO INTERNAZIONALIZZAZIONE  ##################################
################################################################################
import gettext
from gettext import gettext as _
gettext.textdomain('preventares')
################################################################################
############ IMPOSTAZIONI LOGGINGS  ############################################
################################################################################
import logging
LOGGING = "PreventaLib"
logging.getLogger(LOGGING)
################################################################################
############# VARIABILI DI DEFAULT #############################################
################################################################################
# impostazioni di default
default_settings = preventa_print.DEFAULT_SETTINGS

################################################################################
############# DIALOG ###########################################################
################################################################################
class PrintoptionDialog(gtk.Dialog):
    __gtype_name__ = "PrintoptionDialog"

    def __new__(cls, list_capitoli=None, list_categorie=None, 
                print_settings=None):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated PrintoptionDialog object.
        """
        builder = get_builder('PrintoptionDialog')
        new_object = builder.get_object('printoption_dialog')
        new_object.finish_initializing(builder, list_capitoli, list_categorie,
                                       print_settings)
        return new_object

    def __init__(self, list_capitoli=None, list_categorie=None, 
                 print_settings=None):
        """""" 
        pass

    def finish_initializing(self, builder, list_capitoli=None, 
                            list_categorie=None, print_settings=None):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a PrintoptionDialog object with it in order to
        finish initializing the start of the new PrintoptionDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.builder.connect_signals(self)
        self.lista_cat = list_categorie
        self.lista_cap = list_capitoli
        # prepara la lista dei capitoli e categorie da non stampare
        self.cap_list = [[], [], []]
        self.cat_list = [[], [], []]
        ########################################################################
        # crea le impostazioni di stampa #######################################
        ########################################################################
        self.settings = dict()
        self.print_settings = gtk.PrintSettings()
        self.page_setup = gtk.PageSetup()
        # carica le impostazioni di stampa se valide
        if type(print_settings) == type(gtk.PrintSettings()):
            self.print_settings = print_settings
            for key, item in default_settings.items():
                if  self.print_settings.has_key(key):
                    if type(item) is bool:
                        self.settings[key] = self.print_settings.get_bool(key)
                    elif type(item) is int:
                        self.settings[key] = self.print_settings.get_int(key)
                    elif type(item) is float:
                        self.settings[key] = self.print_settings.get_double(key)
                    elif type(item) is str:
                        self.settings[key] = self.print_settings.get(key)
                    else:
                        self.settings[key] = self.print_settings.get(key)
                else:
                    self.settings[key] = default_settings[key]
        else:
            self.settings = default_settings
        # impostazioni di altre variabili
        if self.print_settings.has_key("print_type"):
            self.tipo_stampa = self.print_settings.get("print_type")
        else: self.tipo_stampa = "computo"
        ########################################################################
        # imposta il stup della pagina #########################################
        ########################################################################
        # ricava le impostazioni della gtk.PageSetup da gtk.PrintSettings
        if self.print_settings.get_orientation() is not None:
            self.page_setup.set_orientation(self.print_settings.get_orientation())
        if self.print_settings.get_paper_size() is not None:
            self.page_setup.set_paper_size_and_default_margins(
                            self.print_settings.get_paper_size())
        ########################################################################
        # Imposta un riferimento agli eltri elementi dell'interfaccia grafica ##
        ########################################################################
        # sezione tab 'STAMPA'
        self.radiobutton_computo = builder.get_object("radiobutton1")
        self.radiobutton_lavorazioni = builder.get_object("radiobutton2")
        self.radiobutton_epu = builder.get_object("radiobutton3")
        self.radiobutton_analisi = builder.get_object("radiobutton4")
        self.radiobutton_riepilogo_cmp = builder.get_object("radiobutton5")
        self.radiobutton_riepilogo_epu = builder.get_object("radiobutton6")
        self.radiobutton_riepilogo_inc_mo = builder.get_object("radiobutton7")
        self.radiobutton_riepilogo_inc_sic = builder.get_object("radiobutton8")
        self.checkbutton_pag_presentazione = builder.get_object("checkbutton1")
        self.checkbutton_immagine = builder.get_object("checkbutton2")
        self.checkbutton_data = builder.get_object("checkbutton3")
        self.checkbutton_stampa_unica = builder.get_object("checkbutton4")
        self.checkbutton_nuova_pag_cap = builder.get_object("checkbutton5")
        self.checkbutton_riepilogo_cat = builder.get_object("checkbutton6")
        self.checkbutton_solo_voci_computo = builder.get_object("checkbutton7")
        self.checkbutton_prezzi = builder.get_object("checkbutton8")
        self.checkbutton_desc_estesa = builder.get_object("checkbutton9")
        self.checkbutton_desc_voce = builder.get_object("checkbutton10")
        self.checkbutton_desc_note = builder.get_object("checkbutton11")
        self.checkbutton_report_by_single_cat = builder.get_object("checkbutton12")
        self.checkbutton_start_pages_from = builder.get_object("checkbutton13")
        self.checkbutton_analisi = builder.get_object("checkbutton14")
        self.spinbutton_start_pages_from = builder.get_object("spinbutton6")
        self.entry_titolo = builder.get_object("entry1")
        ########################
        # sezione tab 'GRAFICA'
        ########################
        self.fontbutton1 = builder.get_object("fontbutton1")
        self.fontbutton2 = builder.get_object("fontbutton2")
        self.colorbutton1 = builder.get_object("colorbutton1")
        self.filechooserbutton_img = builder.get_object("filechooserbutton1")
        self.spinbutton_linea_tab = builder.get_object("spinbutton5")
        self.spinbutton_upper_x = builder.get_object("spinbutton1")
        self.spinbutton_upper_y = builder.get_object("spinbutton2")
        self.spinbutton_down_x = builder.get_object("spinbutton3")
        self.spinbutton_down_y = builder.get_object("spinbutton4")
        # pulsanti per tipo di carattere per le descrizioni
        diz_pulsanti = {'b':0, 'i':1, 'u':2, 'big':3, 'small':4}
        pulsanti_descr_codice = ['togglebutton1', 'togglebutton2', 
                                 'togglebutton3', 'togglebutton13', 
                                 'togglebutton14']
        self.togglebutton_descr_codice = [None]*5
        for n in range(5):
            self.togglebutton_descr_codice[n] = builder.get_object(pulsanti_descr_codice[n])
            if n == diz_pulsanti.get(self.settings["description_code_markup"]):
                self.togglebutton_descr_codice[n].set_active(True)
            else:
                self.togglebutton_descr_codice[n].set_active(False)
            self.togglebutton_descr_codice[n].connect("toggled", self.callback_tb_descr, 
                            self.togglebutton_descr_codice, n, "description_code_markup")
        # descrizioni estesa
        pulsanti_descr_estesa = ['togglebutton4', 'togglebutton5', 
                                 'togglebutton6', 'togglebutton18', 
                                 'togglebutton15']
        self.togglebutton_descr_estesa = [None]*5
        for n in range(5):
            self.togglebutton_descr_estesa[n] = builder.get_object(pulsanti_descr_estesa[n])
            if n == diz_pulsanti.get(self.settings["description_extended_markup"]):
                self.togglebutton_descr_estesa[n].set_active(True)
            else:
                self.togglebutton_descr_estesa[n].set_active(False)
            self.togglebutton_descr_estesa[n].connect("toggled", self.callback_tb_descr, 
                            self.togglebutton_descr_estesa, n, "description_extended_markup")
        # descrizioni voce
        pulsanti_descr_voce = ['togglebutton7', 'togglebutton8', 
                               'togglebutton9', 'togglebutton19', 
                               'togglebutton16']
        self.togglebutton_descr_voce = [None]*5
        for n in range(5):
            self.togglebutton_descr_voce[n] = builder.get_object(pulsanti_descr_voce[n])
            if n == diz_pulsanti.get(self.settings["description_voice_markup"]):
                self.togglebutton_descr_voce[n].set_active(True)
            else:
                self.togglebutton_descr_voce[n].set_active(False)
            self.togglebutton_descr_voce[n].connect("toggled", self.callback_tb_descr, 
                            self.togglebutton_descr_voce, n, "description_voice_markup")
        # note
        pulsanti_descr_note = ['togglebutton10', 'togglebutton11', 
                               'togglebutton12', 'togglebutton20', 
                               'togglebutton17']
        self.togglebutton_descr_note = [None]*5
        for n in range(5):
            self.togglebutton_descr_note[n] = builder.get_object(pulsanti_descr_note[n])
            if n == diz_pulsanti.get(self.settings["description_note_markup"]):
                self.togglebutton_descr_note[n].set_active(True)
            else:
                self.togglebutton_descr_note[n].set_active(False)
            self.togglebutton_descr_note[n].connect("toggled", self.callback_tb_descr, 
                            self.togglebutton_descr_note, n, "description_note_markup")
        # elementi analisi
        pulsanti_descr_analisi = ['togglebutton21', 'togglebutton22', 
                                  'togglebutton23', 'togglebutton24', 
                                  'togglebutton25']
        self.togglebutton_descr_analisi = [None]*5
        for n in range(5):
            self.togglebutton_descr_analisi[n] = builder.get_object(pulsanti_descr_analisi[n])
            if n == diz_pulsanti.get(self.settings["description_analysis_markup"]):
                self.togglebutton_descr_analisi[n].set_active(True)
            else:
                self.togglebutton_descr_analisi[n].set_active(False)
            self.togglebutton_descr_analisi[n].connect("toggled", self.callback_tb_descr, 
                            self.togglebutton_descr_analisi, n, "description_analysis_markup")
        # pulsante per impostazioni orientamento e dim. pagina
        self.button_page_setup = builder.get_object('button3')
        self.button_page_setup.connect("clicked", self.callback_bt_page_setup)
        ##################################
        # sezione tab 'ALTRE IMPOSTAZIONI'
        ##################################
        self.treeview_category = builder.get_object("treeview1")
        self.treeview_chapter = builder.get_object("treeview2")
        # imposta i liststore e le treeview CATEGORIE
        self.liststore_category = gtk.ListStore(bool, str, str, str, str)
        self.liststore_chapter = gtk.ListStore(bool, str, str, str, str)
        # popola i liststore
        if self.lista_cat is not None:
            for cat in self.lista_cat[0]:
                n, nome, desc, aum = cat
                self.liststore_category.append((True, n, 'SUP', nome, desc))
            for cat in self.lista_cat[1]:
                n, nome, desc, aum = cat
                self.liststore_category.append((True, n, 'CAT', nome, desc))
            for cat in self.lista_cat[2]:
                n, nome, desc, aum = cat
                self.liststore_category.append((True, n, 'SUB', nome, desc))
        if self.lista_cap is not None:
            for cat in self.lista_cap[0]:
                n, nome, desc, aum = cat
                self.liststore_chapter.append((True, n, 'SUP', nome, desc))
            for cat in self.lista_cap[1]:
                n, nome, desc, aum = cat
                self.liststore_chapter.append((True, n, 'CAP', nome, desc))
            for cat in self.lista_cap[2]:
                n, nome, desc, aum = cat
                self.liststore_chapter.append((True, n, 'SUB', nome, desc))
        # imposta e crea le treeview
        self.crea_treeview_cat (self.treeview_category, self.liststore_category)
        self.treeview_category.set_sensitive(False)
        self.crea_treeview_cat (self.treeview_chapter, self.liststore_chapter)
        self.treeview_chapter.set_sensitive(False)
        # radiobutton selezione categorie e capitoli
        self.radiobutton_all_category = builder.get_object("radiobutton9")
        self.radiobutton_select_category = builder.get_object("radiobutton10")
        self.radiobutton_select_category.connect("toggled", self.cb_activate_treeview,
                                                 self.treeview_category)
        self.radiobutton_all_chapter = builder.get_object("radiobutton11")
        self.radiobutton_select_chapter = builder.get_object("radiobutton12")
        self.radiobutton_select_chapter.connect("toggled", self.cb_activate_treeview,
                                                 self.treeview_chapter)
        ########################################################################
        # Imposta gli eltri elementi dell'interfaccia grafica secondo i settaggi salvati
        ########################################################################
        # sezione tab 'STAMPA'
        if self.tipo_stampa == "computo":
            self.radiobutton_computo.set_active(True)
        elif self.tipo_stampa == "lavorazioni":
            self.radiobutton_lavorazioni.set_active(True)
        elif self.tipo_stampa == "epu":
            self.radiobutton_epu.set_active(True)
        elif self.tipo_stampa == "analisi":
            self.radiobutton_analisi.set_active(True)
        elif self.tipo_stampa == "report-cmp":
            self.radiobutton_riepilogo_cmp.set_active(True)
        elif self.tipo_stampa == "report-epu":
            self.radiobutton_riepilogo_epu.set_active(True)
        elif self.tipo_stampa == "report-inc_mo":
            self.radiobutton_riepilogo_inc_mo.set_active(True)
        elif self.tipo_stampa == "report-inc_sic":
            self.radiobutton_riepilogo_inc_sic.set_active(True)
        self.checkbutton_analisi.set_active(self.settings["print_analisi"])
        self.checkbutton_pag_presentazione.set_active(self.settings["print_first_page"])
        self.checkbutton_immagine.set_active(self.settings["print_logo"])
        self.checkbutton_data.set_active(self.settings["print_date"])
        self.checkbutton_stampa_unica.set_active(self.settings["print_without_chapter"])
        self.checkbutton_nuova_pag_cap.set_active(self.settings["new_chapter_change_page"])
        self.checkbutton_riepilogo_cat.set_active(not self.settings["print_only_grand_total"])
        self.checkbutton_solo_voci_computo.set_active(self.settings["print_only_if_quantity_not_0"])
        self.checkbutton_prezzi.set_active(self.settings["computo_print_price"])
        self.checkbutton_desc_estesa.set_active(self.settings["computo_print_descr_estesa"])
        self.checkbutton_desc_voce.set_active(self.settings["computo_print_descr_voce"])
        self.checkbutton_desc_note.set_active(self.settings["computo_print_note"])
        self.checkbutton_report_by_single_cat.set_active(self.settings["print_report_by_chapter"])
        self.checkbutton_start_pages_from.connect("toggled", self.callback_button,
                                                  self.spinbutton_start_pages_from)
        self.checkbutton_immagine.connect("toggled", self.callback_button, 
                                          self.filechooserbutton_img)
        self.filechooserbutton_img.set_sensitive(self.checkbutton_immagine.get_active())
        if self.settings["add_to_pages_number"] == 0:
            self.checkbutton_start_pages_from.set_active(False)
            self.spinbutton_start_pages_from.set_sensitive(False)
        elif self.settings["add_to_pages_number"] > 0:
            self.spinbutton_start_pages_from.set_value(int(self.settings["add_to_pages_number"]))
        # IMPOSTA IL TESTO DEL TITOLO DI DEFAULT
        self.entry_titolo.set_text(self.settings["main_title"])
        ########################
        # sezione tab 'GRAFICA'
        ########################
        # imposta il font di stampa
        self.fontbutton1.set_font_name(self.settings["font_type"])
        # imposta il font per la prima pagina
        self.fontbutton2.set_font_name(self.settings["font_type_title"])
        # imposta lo spessore della linea della tabella
        self.spinbutton_linea_tab.set_value(float(self.settings["grid_line_width"]))
        # filtro per i file immagine PNG
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        filter1 = gtk.FileFilter()
        filter1.set_name("File PNG")
        filter1.add_mime_type("image/png")
        filter1.add_pattern("*.png")
        self.filechooserbutton_img.add_filter(filter)
        self.filechooserbutton_img.add_filter(filter1)
        self.filechooserbutton_img.set_filter(filter1)
        self.filechooserbutton_img.set_current_folder(get_user_path())
        self.filechooserbutton_img.add_shortcut_folder(get_data_path())
        # impostazione del filename dell'immagine di intestazione
        if self.settings["owner_image"] is not None:
            path = self.settings["owner_image"]
        else: path = ""
        if os.path.isfile(path): # is it file?
            self.filechooserbutton_img.set_filename(path)
            self.filechooserbutton_img.select_filename(path)
        # impostazione offset di stampa
        self.spinbutton_upper_x.set_value(int(self.settings["page_offset_top_x"]))
        self.spinbutton_upper_y.set_value(int(self.settings["page_offset_top_y"]))
        self.spinbutton_down_x.set_value(int(self.settings["page_offset_bottom_x"]))
        self.spinbutton_down_y.set_value(int(self.settings["page_offset_bottom_y"]))
        # imposta il colore del widget per la scelta del colore
        color = gtk.gdk.Color(self.settings["grid_color"])
        self.colorbutton1.set_color(color)
        # imposta il testo del pulsant di modifica del formato e orientamento di stampa
        self.cb_set_label_bt_page_setup()
################################################################################
###  SIGNAL CALLBACKS ##########################################################
################################################################################
    def callback_button(self, widget, data=None):
        data.set_sensitive(widget.get_active())

    def callback_tb_descr(self, widget, togglebuttons, n, setting):
        markup = ['b','i','u','big','small']
        if widget.get_active():
            self.settings[setting] = markup[n]
            for i in range(5):
                if i != n and togglebuttons[i] is not None:
                    togglebuttons[i].set_active(False)
        if not widget.get_active():
            if not togglebuttons[0].get_active() and \
               not togglebuttons[1].get_active() and \
               not togglebuttons[2].get_active() and \
               not togglebuttons[3].get_active() and \
               not togglebuttons[4].get_active():
                    self.settings[setting] = ''
    def callback_tipo_stampa(self, widget, data=None):
        self.entry_titolo.set_text(preventa_print.DEFAULT_TITLE.get(self.tipo_stampa, ""))

    def callback_bt_page_setup(self, widget, data=None):
        window = self.builder.get_object("printoption_dialog")
        self.page_setup = gtk.print_run_page_setup_dialog(window, self.page_setup, 
                                                          self.print_settings)
        # passa le impostazioni della gtk.PageSetup a gtk.PrintSettings
        self.print_settings.set_orientation(self.page_setup.get_orientation())
        self.print_settings.set_paper_size(self.page_setup.get_paper_size())
        #aggiorna l'interfaccia grafica
        self.cb_set_label_bt_page_setup()

    def cb_set_label_bt_page_setup(self, data=None):
        """imposta il testo del pulsante di modifica del formato e orientamento di stampa"""
        orientation = self.print_settings.get('orientation')
        paper_size = self.print_settings.get('paper-format')
        text = _("Formato: <i>{0}</i> - Orientamento: <i>{1}</i>".format(
                 paper_size, orientation))
        self.button_page_setup.set_label(text.replace('None', 'default'))
        self.button_page_setup.child.set_use_markup(True)
        
    def cb_activate_treeview(self, widget, data=None):
        if widget.get_active():
            data.set_sensitive(True)
        elif not widget.get_active():
            data.set_sensitive(False)

    def crea_treeview_cat (self, treeview_obj, liststore):
        """Funzione per creare la treeview dei capitoli e categorie"""
        treeview_obj.liststore = liststore
        treeview_obj.set_model(treeview_obj.liststore)
        treeview_obj.columns = [None]*5
        treeview_obj.columns[0] = gtk.TreeViewColumn('Stampa')
        treeview_obj.columns[1] = gtk.TreeViewColumn('No.')
        treeview_obj.columns[2] = gtk.TreeViewColumn('Tipo')
        treeview_obj.columns[3] = gtk.TreeViewColumn('Nome')
        treeview_obj.columns[4] = gtk.TreeViewColumn('Descrizione')
        for n in range(5):
            # add columns to treeview
            treeview_obj.append_column(treeview_obj.columns[n])
            # create a CellRenderers to render the data
            if n == 0:
                treeview_obj.columns[n].cell = gtk.CellRendererToggle()
            else:
                treeview_obj.columns[n].cell = gtk.CellRendererText()
            # add the cells to the columns
            treeview_obj.columns[n].pack_start(treeview_obj.columns[n].cell, True)
            # make treeview searchable
            treeview_obj.set_search_column(n)
            # set only the cell 0 as editable
            if n == 0:
                treeview_obj.columns[n].cell.set_property("activatable", True)
                treeview_obj.columns[n].add_attribute(treeview_obj.columns[n].cell, "active", 0)
                treeview_obj.columns[n].cell.connect('toggled', self.cell_toggled_cb, liststore)
            else:
                # set the cell attributes to the appropriate liststore column
                treeview_obj.columns[n].set_attributes(treeview_obj.columns[n].cell, text=n)
        
    def cell_toggled_cb(self, cell, path, model):
        """
        Sets the toggled state on the toggle button to true or false.
        """
        model[path][0] = not model[path][0]
        logging.debug("Toggle '{0}' to: {1}".format(model[path][3], model[path][0]))
        return
################################################################################
###  PULSANTI OK / ANNULLA #####################################################
################################################################################
    def ok(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns gtk.RESONSE_OK from run().
        """
        # sezione tab 'STAMPA'
        if self.radiobutton_computo.get_active():
            self.tipo_stampa = "computo"
        elif self.radiobutton_lavorazioni.get_active():
            self.tipo_stampa = "lavorazioni"
        elif self.radiobutton_epu.get_active():
            self.tipo_stampa = "epu"
        elif self.radiobutton_analisi.get_active():
            self.tipo_stampa = "analisi"
        elif self.radiobutton_riepilogo_cmp.get_active():
            self.tipo_stampa = "report-cmp"
        elif self.radiobutton_riepilogo_epu.get_active():
            self.tipo_stampa = "report-epu"
        elif self.radiobutton_riepilogo_inc_mo.get_active():
            self.tipo_stampa = "report-inc_mo"
        elif self.radiobutton_riepilogo_inc_sic.get_active():
            self.tipo_stampa = "report-inc_sic"
        self.settings["print_analisi"] = self.checkbutton_analisi.get_active()
        self.settings["print_first_page"] = self.checkbutton_pag_presentazione.get_active()
        self.settings["print_logo"] = self.checkbutton_immagine.get_active()
        self.settings["print_date"] = self.checkbutton_data.get_active()
        self.settings["print_without_chapter"] = self.checkbutton_stampa_unica.get_active()
        self.settings["new_chapter_change_page"] = self.checkbutton_nuova_pag_cap.get_active()
        self.settings["print_only_grand_total"] = not self.checkbutton_riepilogo_cat.get_active()
        self.settings["print_only_if_quantity_not_0"] = self.checkbutton_solo_voci_computo.get_active()
        self.settings["computo_print_price"] = self.checkbutton_prezzi.get_active()
        self.settings["computo_print_descr_estesa"] = self.checkbutton_desc_estesa.get_active()
        self.settings["computo_print_descr_voce"] = self.checkbutton_desc_voce.get_active()
        self.settings["computo_print_note"] = self.checkbutton_desc_note.get_active()
        self.settings["print_report_by_chapter"] = self.checkbutton_report_by_single_cat.get_active()
        if self.checkbutton_start_pages_from.get_active():
            self.settings["add_to_pages_number"] = -1 + self.spinbutton_start_pages_from.get_value_as_int()              
        else:
            self.settings["add_to_pages_number"] = 0
        # sezione tab 'GRAFICA'
        self.settings["font_type"] = self.fontbutton1.get_font_name()
        self.settings["font_type_title"] = self.fontbutton2.get_font_name()
        # impostazione file immagine
        self.settings["owner_image"] = self.filechooserbutton_img.get_filename()
        # impostazione del colore
        self.settings["grid_color"] = str(self.colorbutton1.get_color())
        self.settings["grid_line_width"] = float(self.spinbutton_linea_tab.get_value())
        # impostazione offset di stampa
        self.settings["page_offset_top_x"] = self.spinbutton_upper_x.get_value_as_int()
        self.settings["page_offset_top_y"] = self.spinbutton_upper_y.get_value_as_int()
        self.settings["page_offset_bottom_x"] = self.spinbutton_down_x.get_value_as_int()
        self.settings["page_offset_bottom_y"] = self.spinbutton_down_y.get_value_as_int()
        # impostazione del titolo
        self.settings["main_title"] = self.entry_titolo.get_text()
        # ricava la lista dei capitoli e delle categorie da stampare
        if not self.radiobutton_all_category.get_active():
            self.liststore_category.foreach(self.foreach_model_cb, self.cat_list)
        else:
            self.cat_list = None
        if not self.radiobutton_all_chapter.get_active():
            self.liststore_chapter.foreach(self.foreach_model_cb, self.cap_list)
        else:
            self.cap_list = None
        # salva le altre impostazioni di stampa
        for key, item in self.settings.items():
            if type(item) is bool:
                self.print_settings.set_bool(key, item)
            if type(item) is int:
                self.print_settings.set_int(key, item)
            if type(item) is float:
                self.print_settings.set_double(key, item)
            if type(item) is str:
                self.print_settings.set(key, item)
        # salva la variabile del tipo di stampa
        self.print_settings.set("print_type", self.tipo_stampa)
        # passa le impostazioni della gtk.PageSetup a gtk.PrintSettings
        self.print_settings.set_orientation(self.page_setup.get_orientation())
        self.print_settings.set_paper_size(self.page_setup.get_paper_size())
        # visualizza informazioni in modalità debug e restituisce le opzioni
        self.print_settings.foreach(self.foreach_cb, user_data=None)
        # concludi
        return self.print_settings, self.cat_list, self.cap_list

    def cancel(self, widget, data=None):
        """The user has elected cancel changes.
        Called before the dialog returns gtk.RESPONSE_CANCEL for run()
        """
        self.cat_list = None
        self.cap_list = None

    def foreach_cb(self, key, value, data):
        logging.debug("PRINT_OPTION: {0} = {1}".format(key, value))
        
    def foreach_model_cb (self, model, path, iter, data=None):
        row = model[path]
        cat = (int(row[1]), row[3], row[4], 0.0)
        # crea la blacklist dei capitoli e categorie (da non stampare)
        if data is not None and row[0]:
            if row[2] == "SUP":
                data[0].append(cat)
            elif row[2] == "SUB":
                data[2].append(cat)
            else:
                data[1].append(cat)
            # elenca gli elementi da stampare
            logging.debug("CHAPTER_TO_PRINT: {0} n°{1} {2}: {3}".format(
                          row[2], row[1], row[3], row[0]))
################################################################################
###  MODIFICA INTERFACCIA PER FUNZIONI DI ESPORTAZIONE #########################
################################################################################
    def modify_ui(self):
        """
        Funzione da richiamare prima di run() per nascondere parti dell'interfaccia
        grafica inutile per le funzioni di esportazione di file (es. in CSV)
        """
        notebook = self.builder.get_object("notebook1")
        frame_titolo = self.builder.get_object("frame3")
        frame_grafica = self.builder.get_object("expander3")
        frame_margini = self.builder.get_object("expander2")
        frame_descrizioni = self.builder.get_object("frame5")
        frame_formato_pagina = self.builder.get_object("frame8")
        frame_titolo.hide()
        frame_grafica.hide()
        frame_margini.hide()
        frame_formato_pagina.hide()
        # del frame intestazioni
        self.builder.get_object("checkbutton1").hide()
        self.builder.get_object("checkbutton2").hide()
        self.builder.get_object("checkbutton3").hide()
        self.builder.get_object("checkbutton5").hide()
        self.builder.get_object("hbox5").hide()
        # del frame descrizioni
        for i in [1,2,3,4,6]:
            self.builder.get_object("hbox%s" % i).hide()
        self.builder.get_object("label18").hide()
        self.builder.get_object("label20").hide()
        #notebook.remove_page(1)
        #notebook.remove_page(-1)
